#pragma once

#include "caosdb/filestreaming/FileLock.h" // for FileMutex, FileWriteLock
#include <boost/filesystem/path.hpp>       // for path
#include <fstream>                         // for ofstream
#include <memory>                          // for shared_ptr
#include <string>                          // for string

namespace FileExchange {

class FileWriter final {
public:
  FileWriter(boost::filesystem::path filename);
  FileWriter(boost::filesystem::path filename,
             std::shared_ptr<FileMutex> mutexPtr);

  ~FileWriter() = default;

  FileWriter(const FileWriter &) = delete;
  FileWriter &operator=(const FileWriter &) = delete;

  FileWriter(FileWriter &&) = default;
  FileWriter &operator=(FileWriter &&) = default;

  void write(const std::string &buffer);

private:
  void openFile();

  std::ofstream stream_;
  boost::filesystem::path filename_;

  std::shared_ptr<FileMutex> mutexPtr_;
  FileWriteLock lock_;
};

} // namespace FileExchange
